@@ -1192,7 +1192,7 @@ describe("any", function() {
         var deferreds = [Q.defer(), Q.defer()];
         var promises = [deferreds[0].promise, deferreds[1].promise];
 
-        return testReject(promises, deferreds);
+        return testReject(promises, deferreds, new Error("Rejected"));
     });
 
     it("rejects after all promises in a sparse array are rejected", function() {
@@ -1201,24 +1201,47 @@ describe("any", function() {
         promises[0] = deferreds[0].promise;
         promises[3] = deferreds[1].promise;
 
-        return testReject(promises, deferreds);
+        return testReject(promises, deferreds, new Error("Rejected"));
     });
 
-    function testReject(promises, deferreds) {
+
+    it("rejects after all promises are rejected with null", function() {
+        var deferreds = [Q.defer(), Q.defer()];
+        var promises = [deferreds[0].promise, deferreds[1].promise];
+
+        return testReject(promises, deferreds, null);
+    });
+
+    it("rejects after all promises are rejected with undefined", function() {
+        var deferreds = [Q.defer(), Q.defer()];
+        var promises = [deferreds[0].promise, deferreds[1].promise];
+
+        return testReject(promises, deferreds, undefined);
+    });
+
+    function testReject(promises, deferreds, rejectionValue) {
         var promise = Q.any(promises);
+        var expectedError;
+
+        if (rejectionValue) {
+          expectedError = new Error(rejectionValue.message);
+        } else {
+          expectedError = new Error("" + rejectionValue);
+        }
 
         for (var index = 0; index < deferreds.length; index++) {
             var deferred = deferreds[index];
             (function() {
-                deferred.reject(new Error('Rejected'));
+              deferred.reject(rejectionValue);
             })();
         }
 
         return Q.delay(250)
           .then(function() {
               expect(promise.isRejected()).toBe(true);
+              expect(promise.inspect().reason).toEqual(expectedError);
               expect(promise.inspect().reason.message)
-              .toBe("Can't get fulfillment value from any promise, all promises were rejected.");
+                .toBe("Q can't get fulfillment value from any promise, all promises were rejected. Last error message: " + expectedError.message);
           })
           .timeout(1000);
     }
@@ -1671,6 +1694,38 @@ describe("fin", function () {
 
     });
 
+    describe("when the callback is invalid", function () {
+        describe("undefined", function () {
+            it("should have a useful error", function () {
+                var foo = {},
+                    threw = false;
+
+                try {
+                    Q().fin(foo.bar);
+                } catch (err) {
+                    expect(err.message).toBe("Q can't apply finally callback");
+                    threw = true;
+                }
+
+                expect(threw).toBe(true);
+            });
+        });
+
+        describe("not a function", function () {
+            it("should have a useful error", function () {
+                var threw = false;
+
+                try {
+                    Q().fin(123);
+                } catch (err) {
+                    expect(err.message).toBe("Q can't apply finally callback");
+                    threw = true;
+                }
+
+                expect(threw).toBe(true);
+            });
+        });
+    });
 });
 
 // Almost like "fin"
@@ -2578,9 +2633,6 @@ if (typeof require === "function") {
             beforeEach(function () {
                 d = domain.create();
             });
-            afterEach(function() {
-                d.dispose();
-            });
 
             it("should work for non-promise async inside a promise handler",
                function (done) {
@@ -2724,6 +2776,72 @@ describe("stack trace formatting", function () {
     });
 });
 
+describe("long stack traces", function () {
+    beforeEach(function () {
+        Q.longStackSupport = true;
+    });
+
+    afterEach(function () {
+        Q.longStackSupport = false;
+    });
+
+    it("include all the calling functions", function () {
+        function func1() {
+            return Q().then(function () { return func2(); });
+        }
+        function func2() {
+            return new Q.Promise(function (resolve, reject) {
+                func3().then(resolve, reject);
+            });
+        }
+        function func3() {
+            return new Q.Promise(function (resolve, reject) {
+                setTimeout(function () {
+                    reject(new Error(REASON));
+                }, 0);
+            });
+        }
+
+        return func1()
+        .catch(function (err) {
+            expect(err.stack).toMatch(/func3(.|\n)*func2(.|\n)*func1/);
+        });
+    });
+
+    it("does not duplicate lines when rethrowing an error", function () {
+        function func1() {
+            return func2()
+                .catch(function rethrow (err) {throw err;})
+                .catch(function rethrow (err) {throw err;});
+        }
+        function func2() {
+            return Q().then(function () {
+                return func3();
+            });
+        }
+        function func3() {
+            return Q.reject(new Error(REASON));
+        }
+
+        return func1()
+        .catch(function (err) {
+            expect(err.stack).toMatch(/func3(.|\n)*func2(.|\n)*func1/);
+            expect(err.stack.match(/func1/g).length).toBe(1);
+        });
+    });
+
+    it("does not add visible properties to thrown errors", function () {
+        return Q().then(function () { throw new Error(REASON); })
+        .catch(function (err) {
+            var keys = [];
+            for (var key in err) {
+                keys.push(key);
+            }
+            expect(keys.length).toBe(0);
+        });
+    });
+});
+
 describe("possible regressions", function () {
 
     describe("gh-9", function () {
